@@ -22,52 +22,48 @@
 #ifndef __G_IREPOSITORY_H__
 #define __G_IREPOSITORY_H__
 
+#define __GIREPOSITORY_H_INSIDE__
+
 #include <glib-object.h>
 #include <gmodule.h>
+#include <gibaseinfo.h>
+#include <gifunctioninfo.h>
+#include <gitypelib.h>
+#include <gitypes.h>
 
 G_BEGIN_DECLS
 
-#define G_TYPE_IREPOSITORY      (g_irepository_get_type ())
-#define G_IREPOSITORY(obj)      (G_TYPE_CHECK_INSTANCE_CAST ((obj), G_TYPE_IREPOSITORY, GIRepository))
+#define G_TYPE_IREPOSITORY              (g_irepository_get_type ())
+#define G_IREPOSITORY(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), G_TYPE_IREPOSITORY, GIRepository))
+#define G_IREPOSITORY_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), G_TYPE_IREPOSITORY, GIRepositoryClass))
+#define G_IS_IREPOSITORY(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), G_TYPE_IREPOSITORY))
+#define G_IS_IREPOSITORY_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), G_TYPE_IREPOSITORY))
+#define G_IREPOSITORY_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), G_TYPE_IREPOSITORY, GIRepositoryClass))
 
-typedef struct _GIRepository         GIRepository; 
-typedef struct _GIRepositoryClass    GIRepositoryClass; 
-typedef struct _GIRepositoryPrivate  GIRepositoryPrivate; 
-typedef struct _GIBaseInfo           GIBaseInfo;
-typedef struct _GICallableInfo       GICallableInfo;
-typedef struct _GIFunctionInfo       GIFunctionInfo;
-typedef struct _GICallbackInfo       GICallbackInfo;
-typedef struct _GIRegisteredTypeInfo GIRegisteredTypeInfo;
-typedef struct _GIStructInfo         GIStructInfo;
-typedef struct _GIUnionInfo          GIUnionInfo;
-typedef struct _GIEnumInfo           GIEnumInfo;
-typedef struct _GIObjectInfo         GIObjectInfo;
-typedef struct _GIInterfaceInfo      GIInterfaceInfo;
-typedef struct _GIConstantInfo       GIConstantInfo;
-typedef struct _GIValueInfo          GIValueInfo;
-typedef struct _GISignalInfo         GISignalInfo;
-typedef struct _GIVFuncInfo          GIVFuncInfo;
-typedef struct _GIPropertyInfo       GIPropertyInfo;
-typedef struct _GIFieldInfo          GIFieldInfo;
-typedef struct _GIArgInfo            GIArgInfo;
-typedef struct _GITypeInfo           GITypeInfo;
-typedef struct _GIErrorDomainInfo    GIErrorDomainInfo;
-typedef struct _GIUnresolvedInfo     GIUnresolvedInfo;
-typedef struct _GTypelib            GTypelib;
+typedef struct _GIRepository         GIRepository;
+typedef struct _GIRepositoryClass    GIRepositoryClass;
+typedef struct _GIRepositoryPrivate  GIRepositoryPrivate;
 
-struct _GIRepository 
-{ 
-  GObject parent; 
+struct _GIRepository
+{
+  GObject parent;
 
   /*< private >*/
   GIRepositoryPrivate *priv;
 };
 
 struct _GIRepositoryClass
-{ 
-  GObjectClass parent; 
+{
+  GObjectClass parent;
 };
 
+/**
+ * GIRepositoryLoadFlags
+ * @G_IREPOSITORY_LOAD_FLAG_LAZY: Load the types lazily.
+ *
+ * Flags that controlls how a typelib is loaded by
+ * GIRepositry, used by g_irepository_load_typelib().
+ */
 typedef enum
 {
   G_IREPOSITORY_LOAD_FLAG_LAZY = 1 << 0
@@ -117,20 +113,16 @@ GOptionGroup * g_irepository_get_option_group (void);
 
 gboolean       g_irepository_dump  (const char *arg, GError **error);
 
-/* Typelib */
-
-GTypelib *   g_typelib_new_from_memory       (guchar       *memory,
-                                                gsize         len);
-GTypelib *   g_typelib_new_from_const_memory (const guchar *memory,
-                                                gsize         len);
-GTypelib *   g_typelib_new_from_mapped_file  (GMappedFile  *mfile);
-void          g_typelib_free                  (GTypelib    *typelib);
-
-gboolean      g_typelib_symbol                (GTypelib    *typelib,
-                                               const gchar *symbol_name,
-                                               gpointer    *symbol);
-const gchar * g_typelib_get_namespace         (GTypelib    *typelib);
-
+/**
+ * GIRepositoryError:
+ * @G_IREPOSITORY_ERROR_TYPELIB_NOT_FOUND: the typelib could not be found.
+ * @G_IREPOSITORY_ERROR_NAMESPACE_MISMATCH: the namespace does not match the
+ * requested namespace.
+ * @G_IREPOSITORY_ERROR_NAMESPACE_VERSION_CONFLICT: the version of the
+ * typelib does not match the requested version.
+ * @G_IREPOSITORY_ERROR_LIBRARY_NOT_FOUND: the library used by the typelib
+ * could not be found.
+ */
 typedef enum
 {
   G_IREPOSITORY_ERROR_TYPELIB_NOT_FOUND,
@@ -153,128 +145,24 @@ void gi_cclosure_marshal_generic (GClosure       *closure,
                                   gpointer        invocation_hint,
                                   gpointer        marshal_data);
 
-/* Types of objects registered in the repository */
-
-typedef enum 
-{
-  GI_INFO_TYPE_INVALID,
-  GI_INFO_TYPE_FUNCTION,
-  GI_INFO_TYPE_CALLBACK,
-  GI_INFO_TYPE_STRUCT,
-  GI_INFO_TYPE_BOXED,
-  GI_INFO_TYPE_ENUM,         /*  5 */
-  GI_INFO_TYPE_FLAGS,
-  GI_INFO_TYPE_OBJECT,
-  GI_INFO_TYPE_INTERFACE,
-  GI_INFO_TYPE_CONSTANT,
-  GI_INFO_TYPE_ERROR_DOMAIN, /* 10 */
-  GI_INFO_TYPE_UNION,
-  GI_INFO_TYPE_VALUE,
-  GI_INFO_TYPE_SIGNAL,
-  GI_INFO_TYPE_VFUNC,
-  GI_INFO_TYPE_PROPERTY,     /* 15 */
-  GI_INFO_TYPE_FIELD,
-  GI_INFO_TYPE_ARG,
-  GI_INFO_TYPE_TYPE,
-  GI_INFO_TYPE_UNRESOLVED
-} GIInfoType;
-
-
-/* GIBaseInfo */
-
-typedef struct {
-  gpointer data;
-  gpointer data2;
-  gpointer data3;
-  gpointer data4;
-} GIAttributeIter;
-
-GIBaseInfo *           g_base_info_ref              (GIBaseInfo   *info);
-void                   g_base_info_unref            (GIBaseInfo   *info);
-GIInfoType             g_base_info_get_type         (GIBaseInfo   *info);
-const gchar *          g_base_info_get_name         (GIBaseInfo   *info);
-const gchar *          g_base_info_get_namespace    (GIBaseInfo   *info);
-gboolean               g_base_info_is_deprecated    (GIBaseInfo   *info);
-const gchar *          g_base_info_get_attribute    (GIBaseInfo   *info,
-                                                     const gchar  *name);
-gboolean               g_base_info_iterate_attributes (GIBaseInfo      *info,
-                                                       GIAttributeIter *iterator,
-                                                       char           **name,
-                                                       char          **value);
-GIBaseInfo *           g_base_info_get_container    (GIBaseInfo   *info);
-GTypelib *             g_base_info_get_typelib      (GIBaseInfo   *info);
-gboolean               g_base_info_equal            (GIBaseInfo   *info1,
-                                                     GIBaseInfo   *info2);
-
-GIBaseInfo *           g_info_new                   (GIInfoType    type,
-						     GIBaseInfo   *container,
-						     GTypelib     *typelib, 
-						     guint32       offset);
-
-
-/* GIFunctionInfo */
-
-typedef enum
-{
-  GI_FUNCTION_IS_METHOD      = 1 << 0,
-  GI_FUNCTION_IS_CONSTRUCTOR = 1 << 1,
-  GI_FUNCTION_IS_GETTER      = 1 << 2,
-  GI_FUNCTION_IS_SETTER      = 1 << 3,
-  GI_FUNCTION_WRAPS_VFUNC    = 1 << 4,
-  GI_FUNCTION_THROWS         = 1 << 5
-} GIFunctionInfoFlags;
-
-const gchar *           g_function_info_get_symbol     (GIFunctionInfo *info);
-GIFunctionInfoFlags     g_function_info_get_flags      (GIFunctionInfo *info);
-GIPropertyInfo *        g_function_info_get_property   (GIFunctionInfo *info);
-GIVFuncInfo *           g_function_info_get_vfunc      (GIFunctionInfo *info);
-
-typedef union 
-{
-  gboolean v_boolean;
-  gint8    v_int8;
-  guint8   v_uint8;
-  gint16   v_int16;
-  guint16  v_uint16;
-  gint32   v_int32;
-  guint32  v_uint32;
-  gint64   v_int64;
-  guint64  v_uint64;
-  gfloat   v_float;
-  gdouble  v_double;
-  gshort   v_short;
-  gushort  v_ushort;
-  gint     v_int;
-  guint    v_uint;
-  glong    v_long;
-  gulong   v_ulong;
-  gssize   v_ssize;
-  gsize    v_size;
-  gchar *  v_string;
-  gpointer v_pointer;
-} GArgument;
-
-#define G_INVOKE_ERROR (g_invoke_error_quark ())
-GQuark g_invoke_error_quark (void);
-
-typedef enum
-{
-  G_INVOKE_ERROR_FAILED,
-  G_INVOKE_ERROR_SYMBOL_NOT_FOUND,
-  G_INVOKE_ERROR_ARGUMENT_MISMATCH
-} GInvokeError;
-
-gboolean              g_function_info_invoke         (GIFunctionInfo *info, 
-						      const GArgument  *in_args,
-						      int               n_in_args,
-						      const GArgument  *out_args,
-						      int               n_out_args,
-						      GArgument        *return_value,
-						      GError          **error);
-
-
 /* GICallableInfo */
 
+#define GI_IS_CALLABLE_INFO(info) \
+    ((g_base_info_get_type((GIBaseInfo*)info) == GI_INFO_TYPE_FUNCTION) || \
+     (g_base_info_get_type((GIBaseInfo*)info) == GI_INFO_TYPE_CALLBACK) || \
+     (g_base_info_get_type((GIBaseInfo*)info) == GI_INFO_TYPE_SIGNAL) || \
+     (g_base_info_get_type((GIBaseInfo*)info) == GI_INFO_TYPE_VFUNC))
+
+/**
+ * GITransfer:
+ * @GI_TRANSFER_NOTHING: transfer nothing to the caller
+ * @GI_TRANSFER_CONTAINER: transfer the container (eg list, array,
+ * hashtable), but no the contents to the caller.
+ * @GI_TRANSFER_EVERYTHING: transfer everything to the caller.
+ *
+ * Represent the transfer ownership information of a #GICallableInfo or
+ * a #GIArgInfo.
+ */
 typedef enum {
   GI_TRANSFER_NOTHING,
   GI_TRANSFER_CONTAINER,
@@ -282,45 +170,112 @@ typedef enum {
 } GITransfer;
 
 GITypeInfo *           g_callable_info_get_return_type (GICallableInfo *info);
+void                   g_callable_info_load_return_type (GICallableInfo *info,
+                                                         GITypeInfo     *type);
 GITransfer             g_callable_info_get_caller_owns (GICallableInfo *info);
 gboolean               g_callable_info_may_return_null (GICallableInfo *info);
 gint                   g_callable_info_get_n_args      (GICallableInfo *info);
 GIArgInfo *            g_callable_info_get_arg         (GICallableInfo *info,
-							gint           n);
+                                                        gint            n);
+void                   g_callable_info_load_arg        (GICallableInfo *info,
+                                                        gint            n,
+                                                        GIArgInfo      *arg);
 
 /* GIArgInfo */
 
+#define GI_IS_ARG_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_ARG)
+
+/**
+ * GIDirection:
+ * @GI_DIRECTION_IN: in argument.
+ * @GI_DIRECTION_OUT: out argument.
+ * @GI_DIRECTION_INOUT: in and out argument.
+ *
+ * The direction of a #GIArgInfo.
+ */
 typedef enum  {
   GI_DIRECTION_IN,
   GI_DIRECTION_OUT,
   GI_DIRECTION_INOUT
 } GIDirection;
 
+/**
+ * GIScopeType:
+ * @GI_SCOPE_TYPE_INVALID: The argument is not of callback type.
+ * @GI_SCOPE_TYPE_CALL: The callback and associated user_data is only
+ * used during the call to this function.
+ * @GI_SCOPE_TYPE_ASYNC: The callback and associated user_data is
+ * only used until the callback is invoked, and the callback.
+ * is invoked always exactly once.
+ * @GI_SCOPE_TYPE_NOTIFIED: The callback and and associated
+ * user_data is used until the caller is notfied via the destroy_notify.
+ *
+ * Scope type of a #GIArgInfo representing callback, determines how the
+ * callback is invoked and is used to decided when the invoke structs
+ * can be freed.
+ */
 typedef enum {
-  GI_SCOPE_TYPE_INVALID, /* The argument is not of callback type */
-  GI_SCOPE_TYPE_CALL, /* The callback and associated user_data is only used during the
-                         call to this function */
-  GI_SCOPE_TYPE_ASYNC, /* The callback and associated user_data is 
-                          only used until the callback is invoked, and the callback 
-                          is invoked always exactly once. */
-  GI_SCOPE_TYPE_NOTIFIED /* The callback and and associated user_data is
-                            used until the caller is notfied via the destroy_notify */
+  GI_SCOPE_TYPE_INVALID,
+  GI_SCOPE_TYPE_CALL,
+  GI_SCOPE_TYPE_ASYNC,
+  GI_SCOPE_TYPE_NOTIFIED
 } GIScopeType;
 
 GIDirection            g_arg_info_get_direction          (GIArgInfo *info);
-gboolean               g_arg_info_is_dipper              (GIArgInfo *info);
 gboolean               g_arg_info_is_return_value        (GIArgInfo *info);
 gboolean               g_arg_info_is_optional            (GIArgInfo *info);
+gboolean               g_arg_info_is_caller_allocates    (GIArgInfo *info);
 gboolean               g_arg_info_may_be_null            (GIArgInfo *info);
 GITransfer             g_arg_info_get_ownership_transfer (GIArgInfo *info);
 GIScopeType            g_arg_info_get_scope              (GIArgInfo *info);
 gint                   g_arg_info_get_closure            (GIArgInfo *info);
 gint                   g_arg_info_get_destroy            (GIArgInfo *info);
 GITypeInfo *           g_arg_info_get_type               (GIArgInfo *info);
-
+void                   g_arg_info_load_type              (GIArgInfo *info,
+                                                          GITypeInfo *type);
 
 /* GITypeInfo */
 
+#define GI_IS_TYPE_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_TYPE)
+
+/**
+ * GITypeTag:
+ * @GI_TYPE_TAG_VOID: void
+ * @GI_TYPE_TAG_BOOLEAN: boolean
+ * @GI_TYPE_TAG_INT8: 8-bit signed integer
+ * @GI_TYPE_TAG_UINT8: 8-bit unsigned integer
+ * @GI_TYPE_TAG_INT16: 16-bit signed integer
+ * @GI_TYPE_TAG_UINT16: 16-bit unsigned integer
+ * @GI_TYPE_TAG_INT32: 32-bit signed integer
+ * @GI_TYPE_TAG_UINT32: 32-bit unsigned integer
+ * @GI_TYPE_TAG_INT64: 64-bit signed integer
+ * @GI_TYPE_TAG_UINT64: 64-bit unsigned integer
+ * @GI_TYPE_TAG_SHORT: signed short
+ * @GI_TYPE_TAG_USHORT: unsigned hosrt
+ * @GI_TYPE_TAG_INT: signed integer
+ * @GI_TYPE_TAG_UINT: unsigned integer
+ * @GI_TYPE_TAG_LONG: signed long
+ * @GI_TYPE_TAG_ULONG: unsigned long
+ * @GI_TYPE_TAG_SSIZE: ssize_t
+ * @GI_TYPE_TAG_SIZE: size_t
+ * @GI_TYPE_TAG_FLOAT: float
+ * @GI_TYPE_TAG_DOUBLE: double floating point
+ * @GI_TYPE_TAG_TIME_T: time_t
+ * @GI_TYPE_TAG_GTYPE: a #GType
+ * @GI_TYPE_TAG_UTF8: a UTF-8 encoded string
+ * @GI_TYPE_TAG_FILENAME: a filename, encoded in the same encoding
+ * as the native filesystem is using.
+ * @GI_TYPE_TAG_ARRAY: an array
+ * @GI_TYPE_TAG_INTERFACE: an extended interface object
+ * @GI_TYPE_TAG_GLIST: a #GList
+ * @GI_TYPE_TAG_GSLIST: a #GSList
+ * @GI_TYPE_TAG_GHASH: a #GHashTable
+ * @GI_TYPE_TAG_ERROR: a #GError
+ *
+ * The type tag of a #GITypeInfo.
+ */
 typedef enum {
   /* Basic types */
   GI_TYPE_TAG_VOID      =  0,
@@ -328,7 +283,7 @@ typedef enum {
   GI_TYPE_TAG_INT8      =  2,
   GI_TYPE_TAG_UINT8     =  3,
   GI_TYPE_TAG_INT16     =  4,
-  GI_TYPE_TAG_UINT16    =  5,  
+  GI_TYPE_TAG_UINT16    =  5,
   GI_TYPE_TAG_INT32     =  6,
   GI_TYPE_TAG_UINT32    =  7,
   GI_TYPE_TAG_INT64     =  8,
@@ -358,6 +313,22 @@ typedef enum {
    * See docs/typelib-format.txt SimpleTypeBlob definition */
 } GITypeTag;
 
+/**
+ * GIArrayType:
+ * @GI_ARRAY_TYPE_C: a C array, char[] for instance
+ * @GI_ARRAY_TYPE_ARRAY: a @GArray array
+ * @GI_ARRAY_TYPE_PTR_ARRAY: a #GPtrArray array
+ * @GI_ARRAY_TYPE_BYTE_ARRAY: a #GByteArray array
+ *
+ * The type of array in a #GITypeInfo.
+ */
+typedef enum {
+  GI_ARRAY_TYPE_C,
+  GI_ARRAY_TYPE_ARRAY,
+  GI_ARRAY_TYPE_PTR_ARRAY,
+  GI_ARRAY_TYPE_BYTE_ARRAY
+} GIArrayType;
+
 #define G_TYPE_TAG_IS_BASIC(tag) (tag < GI_TYPE_TAG_ARRAY)
 
 const gchar*           g_type_tag_to_string            (GITypeTag   type);
@@ -370,6 +341,7 @@ GIBaseInfo *           g_type_info_get_interface       (GITypeInfo *info);
 gint                   g_type_info_get_array_length    (GITypeInfo *info);
 gint                   g_type_info_get_array_fixed_size(GITypeInfo *info);
 gboolean               g_type_info_is_zero_terminated  (GITypeInfo *info);
+GIArrayType            g_type_info_get_array_type      (GITypeInfo *info);
 
 gint                   g_type_info_get_n_error_domains (GITypeInfo *info);
 GIErrorDomainInfo     *g_type_info_get_error_domain    (GITypeInfo *info,
@@ -377,17 +349,34 @@ GIErrorDomainInfo     *g_type_info_get_error_domain    (GITypeInfo *info,
 
 /* GIErrorDomainInfo */
 
+#define GI_IS_ERROR_DOMAIN_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_ERROR_DOMAIN)
+
 const gchar *          g_error_domain_info_get_quark   (GIErrorDomainInfo *info);
 GIInterfaceInfo *           g_error_domain_info_get_codes (GIErrorDomainInfo *info);
 
 
 /* GIValueInfo */
- 
+
+#define GI_IS_VALUE_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_VALUE)
+
 glong                  g_value_info_get_value      (GIValueInfo *info);
 
 
 /* GIFieldInfo */
 
+#define GI_IS_FIELD_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_FIELD)
+
+/**
+ * GIFieldInfoFlags:
+ * @GI_FIELD_IS_READABLE: field is readable.
+ * @GI_FIELD_IS_WRITABLE: field is writable.
+ *
+ * Flags for a #GIFieldInfo.
+ */
+
 typedef enum
 {
   GI_FIELD_IS_READABLE = 1 << 0,
@@ -407,6 +396,10 @@ gboolean g_field_info_set_field (GIFieldInfo     *field_info,
 				 const GArgument *value);
 
 /* GIUnionInfo */
+
+#define GI_IS_UNION_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_UNION)
+
 gint                   g_union_info_get_n_fields  (GIUnionInfo *info);
 GIFieldInfo *          g_union_info_get_field     (GIUnionInfo *info,
 					           gint         n);
@@ -425,6 +418,10 @@ gsize                  g_union_info_get_alignment  (GIUnionInfo *info);
 
 
 /* GIStructInfo */
+
+#define GI_IS_STRUCT_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_STRUCT)
+
 gint                   g_struct_info_get_n_fields  (GIStructInfo *info);
 GIFieldInfo *          g_struct_info_get_field     (GIStructInfo *info,
 						    gint         n);
@@ -436,15 +433,27 @@ GIFunctionInfo *       g_struct_info_find_method   (GIStructInfo *info,
 gsize                  g_struct_info_get_size      (GIStructInfo *info);
 gsize                  g_struct_info_get_alignment (GIStructInfo *info);
 gboolean               g_struct_info_is_gtype_struct (GIStructInfo *info);
+gboolean               g_struct_info_is_foreign    (GIStructInfo *info);
 
 /* GIRegisteredTypeInfo */
 
+#define GI_IS_REGISTERED_TYPE_INFO(info) \
+    ((g_base_info_get_type((GIBaseInfo*)info) == GI_INFO_TYPE_ENUM) || \
+     (g_base_info_get_type((GIBaseInfo*)info) == GI_INFO_TYPE_INTERFACE) || \
+     (g_base_info_get_type((GIBaseInfo*)info) == GI_INFO_TYPE_OBJECT) || \
+     (g_base_info_get_type((GIBaseInfo*)info) == GI_INFO_TYPE_STRUCT) || \
+     (g_base_info_get_type((GIBaseInfo*)info) == GI_INFO_TYPE_UNION))
+
 const gchar *          g_registered_type_info_get_type_name (GIRegisteredTypeInfo *info);
 const gchar *          g_registered_type_info_get_type_init (GIRegisteredTypeInfo *info);
 GType                  g_registered_type_info_get_g_type    (GIRegisteredTypeInfo *info);
 
 /* GIEnumInfo */
 
+#define GI_IS_ENUM_INFO(info) \
+    ((g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_ENUM) || \
+     (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_FLAGS))
+
 gint                   g_enum_info_get_n_values             (GIEnumInfo      *info);
 GIValueInfo  *         g_enum_info_get_value                (GIEnumInfo      *info,
 							     gint            n);
@@ -452,8 +461,11 @@ GITypeTag              g_enum_info_get_storage_type         (GIEnumInfo      *in
 
 /* GIObjectInfo */
 
-const gchar *          g_object_info_get_type_name 	    (GIObjectInfo    *info);
-const gchar *          g_object_info_get_type_init 	    (GIObjectInfo    *info);
+#define GI_IS_OBJECT_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_OBJECT)
+
+const gchar *          g_object_info_get_type_name	    (GIObjectInfo    *info);
+const gchar *          g_object_info_get_type_init	    (GIObjectInfo    *info);
 gboolean               g_object_info_get_abstract           (GIObjectInfo    *info);
 GIObjectInfo *         g_object_info_get_parent             (GIObjectInfo    *info);
 gint                   g_object_info_get_n_interfaces       (GIObjectInfo    *info);
@@ -483,9 +495,12 @@ GIConstantInfo *       g_object_info_get_constant           (GIObjectInfo    *in
 							     gint            n);
 GIStructInfo *         g_object_info_get_class_struct       (GIObjectInfo    *info);
 
-							     
+
 /* GIInterfaceInfo */
 
+#define GI_IS_INTERFACE_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_INTERFACE)
+
 gint                   g_interface_info_get_n_prerequisites (GIInterfaceInfo *info);
 GIBaseInfo *           g_interface_info_get_prerequisite    (GIInterfaceInfo *info,
 							     gint        n);
@@ -496,7 +511,7 @@ gint                   g_interface_info_get_n_methods       (GIInterfaceInfo *in
 GIFunctionInfo *       g_interface_info_get_method          (GIInterfaceInfo *info,
 							     gint        n);
 GIFunctionInfo *       g_interface_info_find_method         (GIInterfaceInfo *info,
-		       				             const gchar *name);
+						             const gchar *name);
 gint                   g_interface_info_get_n_signals       (GIInterfaceInfo *info);
 GISignalInfo *         g_interface_info_get_signal          (GIInterfaceInfo *info,
 							     gint        n);
@@ -514,12 +529,18 @@ GIStructInfo *         g_interface_info_get_iface_struct    (GIInterfaceInfo *in
 
 /* GIPropertyInfo  */
 
+#define GI_IS_PROPERTY_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_PROPERTY)
+
 GParamFlags             g_property_info_get_flags                (GIPropertyInfo         *info);
 GITypeInfo *            g_property_info_get_type                 (GIPropertyInfo         *info);
 
 
 /* GISignalInfo */
 
+#define GI_IS_SIGNAL_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_SIGNAL)
+
 GSignalFlags            g_signal_info_get_flags                  (GISignalInfo           *info);
 GIVFuncInfo *           g_signal_info_get_class_closure          (GISignalInfo           *info);
 gboolean                g_signal_info_true_stops_emit            (GISignalInfo           *info);
@@ -527,6 +548,17 @@ gboolean                g_signal_info_true_stops_emit            (GISignalInfo
 
 /* GIVFuncInfo */
 
+#define GI_IS_VFUNC_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_VFUNC)
+
+/**
+ * GIVFuncInfoFlags:
+ * @GI_VFUNC_MUST_CHAIN_UP: chains up to the parent type
+ * @GI_VFUNC_MUST_OVERRIDE: overrides
+ * @GI_VFUNC_MUST_NOT_OVERRIDE: does not override
+ *
+ * Flags of a #GIVFuncInfo struct.
+ */
 typedef enum
 {
   GI_VFUNC_MUST_CHAIN_UP     = 1 << 0,
@@ -542,6 +574,9 @@ GIFunctionInfo *        g_vfunc_info_get_invoker                 (GIVFuncInfo
 
 /* GIConstantInfo */
 
+#define GI_IS_CONSTANT_INFO(info) \
+    (g_base_info_get_type((GIBaseInfo*)info) ==  GI_INFO_TYPE_CONSTANT)
+
 GITypeInfo *            g_constant_info_get_type                 (GIConstantInfo         *info);
 gint                    g_constant_info_get_value                (GIConstantInfo         *info,
 								  GArgument             *value);
@@ -549,5 +584,6 @@ gint                    g_constant_info_get_value                (GIConstantInfo
 
 G_END_DECLS
 
+
 #endif  /* __G_IREPOSITORY_H__ */
 
